/* vim: set tabstop=4:softtabstop=4:shiftwidth=4:noexpandtab */

/*
 * ridle — wrapper that runs a command the lowest priority possible
 * © Oleksandr Natalenko <oleksandr@natalenko.name>, 2023
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <linux/ioprio.h>
#include <linux/sched.h>
#include <sched.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/resource.h>
#include <sys/syscall.h>
#include <sysexits.h>
#include <unistd.h>

int main(int _argc, char** _argv)
{
	struct sched_param schd_prm;
	char* file = NULL;
	char** argv = NULL;
	char* argvshell[2] = { NULL, };
	bool low;
	int policy;
	int ioprio;

	memset(&schd_prm, 0, sizeof schd_prm);

	low = !strcmp(program_invocation_short_name, "rlow");

	if (_argc == 1)
	{
		file = getenv("SHELL");
		if (!file)
			exit(EX_USAGE);
		argvshell[0] = file;
		argv = argvshell;
	} else
	{
		file = _argv[1];
		argv = &_argv[1];
	}

	if (setpriority(PRIO_PROCESS, 0, PRIO_MAX - 1) == -1)
	{
		perror("setpriority");
		exit(EX_OSERR);
	}

	if (low)
		policy = SCHED_OTHER;
	else
		policy = SCHED_IDLE;

	if (sched_setscheduler(0, policy, &schd_prm) == -1)
	{
		perror("sched_setscheduler");
		exit(EX_OSERR);
	}

    if (low)
		ioprio = IOPRIO_PRIO_VALUE(IOPRIO_CLASS_BE, IOPRIO_BE_NR - 1);
	else
		ioprio = IOPRIO_PRIO_VALUE(IOPRIO_CLASS_IDLE, 0);

	if (syscall(SYS_ioprio_set, IOPRIO_WHO_PROCESS, 0, ioprio) == -1)
	{
		perror("ioprio_set");
		exit(EX_OSERR);
	}

	if (execvp(file, argv) == -1)
	{
		perror("execvp");
		exit(EX_SOFTWARE);
	}
}
